unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DirectShow9, StdCtrls, MMSystem, SLStreamTypes,
  SLComponentCollection, LPDrawLayers, SLScope;

const WM_NEW_BUFFER = WM_APP + 100;

type
  TSampleGrabberCB = class;

  TForm1 = class(TForm)
    StartButton: TButton;
    StopButton: TButton;
    SLScope1: TSLScope;
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FGraph              : IGraphBuilder;
    FMediaControl       : IMediaControl;
    FAudioSampleGrabber : ISampleGrabber;
    FAudioGrabberCB     : TSampleGrabberCB;

  private
    function  ConnectFilterToPin( ASourceFilter : IBaseFilter; ASinkPin : IPin ) : Boolean;
    
  private
    procedure WMNewBuffer(var Message: TMessage); message WM_NEW_BUFFER;
    
  public
    function SampleCB(SampleTime: Double; pSample: IMediaSample): HResult; stdcall;
    
  end;

  TSampleGrabberCB = class(TObject, IUnknown, ISampleGrabberCB)
  private
    FForm : TForm1;

  public
    // IUnknown
    function QueryInterface(const IID: TGUID; out Obj): HRESULT; stdcall;
    function _AddRef() : Integer; stdcall;
    function _Release() : Integer; stdcall;
    // ISampleGrabberCB
    function SampleCB(SampleTime: Double; pSample: IMediaSample): HResult; stdcall;
    function BufferCB(SampleTime: Double; pBuffer: PByte; BufferLen: longint): HResult; stdcall;
    
  public

    constructor Create( AForm : TForm1 );
    
  end;
  
var
  Form1: TForm1;

implementation

{$R *.dfm}

uses ActiveX;

var GCriticalSection  : TRTLCriticalSection;
var GHMem             : HGLOBAL;
var GFrameBuffer      : Pointer;
var GBufferLen        : Integer;
var GFormHandle       : HWND;

function DSGetPin( AFilter : IBaseFilter; APinDir : TPinDirection ) : IPin;
var
  hr          : HResult;
  APinEnum    : IEnumPins;
  Fetched     : ULONG;
  ADir        : TPinDirection;

begin
  Result := NIL;
  hr := AFilter.EnumPins( APinEnum );
  if( Failed( hr )) then
    Exit;

  while( Succeeded( APinEnum.Next( 1, Result, @Fetched ) ) ) do
    begin
    if( Fetched = 0 ) then
      Break;

    hr := Result.QueryDirection( ADir );
    if( Failed( hr )) then
      Continue;

    if( ADir = APinDir ) then
      Exit;

    end;

  Result := NIL;
end;

procedure DSFreeMediaType(mt: TAMMediaType);
begin
  if( mt.cbFormat <> 0 ) then
    begin
    CoTaskMemFree(mt.pbFormat);
    // Strictly unnecessary but tidier
    mt.cbFormat := 0;
    mt.pbFormat := NIL;
    end;

  mt.pUnk := NIL;
end;

procedure TForm1.StartButtonClick(Sender: TObject);
var
  WideFileName  : WideString;
  AFileFilter   : IBaseFilter;
  AGrabber      : IBaseFilter;
  hr            : HResult;
  AMediaType    : TAMMediaType;
  AOutPin       : IPin;
  AInPin        : IPin;
  lpFormat      : PWaveFormat;

begin
  GFormHandle := Handle;
  
  FAudioGrabberCB := TSampleGrabberCB.Create( Self );

  CoCreateInstance( CLSID_FilterGraph, NIL, CLSCTX_INPROC, IGraphBuilder, FGraph );
  WideFileName := 'Demo.wav';
  hr := FGraph.AddSourceFilter( @( WideFileName[ 1 ] ), 'File Source Filter', AFileFilter );

  // Create a grabber
  CoCreateInstance(CLSID_SampleGrabber, NIL, CLSCTX_INPROC, IBaseFilter, AGrabber );
  FGraph.AddFilter( AGrabber, 'Audio Grabber' );

  FAudioSampleGrabber := AGrabber as ISampleGrabber;
  FAudioSampleGrabber.SetBufferSamples(False);
  FAudioSampleGrabber.SetOneShot(False);
  FAudioSampleGrabber.SetCallback( FAudioGrabberCB, 0);

  // Set the grabber's Media Type
  ZeroMemory(@AMediaType, SizeOf(AMediaType));
  AMediaType.formattype := FORMAT_WaveFormatEx;
  AMediaType.majortype  := MEDIATYPE_Audio;
  AMediaType.subtype := MEDIASUBTYPE_PCM;

  FAudioSampleGrabber.SetMediaType( AMediaType );

  AInPin := DSGetPin( AGrabber, PINDIR_INPUT );

  ConnectFilterToPin( AFileFilter, AInPin );
  
  FAudioSampleGrabber.GetConnectedMediaType( AMediaType );

  lpFormat := PWaveFormat( AMediaType.pbFormat );

  DSFreeMediaType( AMediaType );

  AOutPin := DSGetPin( AGrabber, PINDIR_OUTPUT );

  hr := FGraph.Render( AOutPin );

  FGraph.QueryInterface( IMediaControl, FMediaControl );

  FMediaControl.Run();

  StartButton.Enabled := False;
  StopButton.Enabled := True;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  if( FMediaControl = NIL ) then
    Exit;
    
  FMediaControl.Stop();
  
  StartButton.Enabled := True;
  StopButton.Enabled := False;

  FMediaControl := NIL;
  FAudioSampleGrabber := NIL;
  FGraph := NIL;
  FAudioGrabberCB.Free();

  if( GFrameBuffer <> NIL ) then
    begin
    // Release the old frame buffer if present.
    GlobalUnlock( GHMem );
    GlobalFree( GHMem );
    end;

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  StopButtonClick( Sender );
end;

//-----------------------------------------------------------------
function TForm1.SampleCB(SampleTime: Double; pSample: IMediaSample): HResult;
var
  ABufferLen : Cardinal;
  pBuffer    : PByte;

begin
  EnterCriticalSection( GCriticalSection );

  if( GFormHandle <> 0 ) then
    begin
    if( GFrameBuffer <> NIL ) then
      begin
      // Release the old frame buffer if present.
      GlobalUnlock( GHMem );
      GlobalFree( GHMem );
      end;

    ABufferLen := pSample.GetActualDataLength(); //GetSize();

    // Allocate new frame buffer.
    GHMem := GlobalAlloc( GMEM_FIXED, ABufferLen );
    GFrameBuffer := GlobalLock( GHMem );

    pSample.GetPointer( pBuffer );
    
    // Copy the data to the frame buffer.
    CopyMemory( GFrameBuffer, pBuffer, ABufferLen );
    GBufferLen := ABufferLen;

    LeaveCriticalSection( GCriticalSection );
    PostMessage( GFormHandle, WM_NEW_BUFFER, 0, 0 );
    end;

  Result := 1;
end;
//-----------------------------------------------------------------
procedure TForm1.WMNewBuffer(var Message: TMessage);
var
  ABuffer : array of Integer;
  I       : Integer;
  APtr    : PSmallInt;

begin
  EnterCriticalSection( GCriticalSection );

  // Copy the pixel data to the bitmap.
  SetLength( ABuffer, GBufferLen div 4 );
  Move( GFrameBuffer^, ABuffer[ 0 ], GBufferLen );
  
  LeaveCriticalSection( GCriticalSection );

  // Show the data in a Scope
  SLScope1.Channels[ 0 ].Data.Clear();
  SLScope1.Channels[ 1 ].Data.Clear();
  for I := 0 to GBufferLen div 4 - 1 do
    begin
    APtr := PSmallInt( @ABuffer[ I ] ); 
    SLScope1.Channels[ 0 ].Data.AddYPoint( APtr^ );
    Inc( APtr );
    SLScope1.Channels[ 1 ].Data.AddYPoint( APtr^ );
    end;
     
end;
//---------------------------------------------------------------------------
function  TForm1.ConnectFilterToPin( ASourceFilter : IBaseFilter; ASinkPin : IPin ) : Boolean;
var
  hr          : HResult;
  AOutPin     : IPin;
  APinEnum    : IEnumPins;
  APinDir     : TPinDirection;
  Fetched     : ULONG;

begin
  Result := False;
  hr := ASourceFilter.EnumPins( APinEnum );
  if( Failed( hr )) then
    Exit;

  while( Succeeded( APinEnum.Next( 1, AOutPin, @Fetched ) ) ) do
    begin
    if( Fetched = 0 ) then
      Exit;

    hr := AOutPin.QueryDirection( APinDir );
    if( Failed( hr )) then
      Continue;

    if( APinDir = PINDIR_OUTPUT ) then
      begin
      hr := FGraph.Connect( AOutPin, ASinkPin );
      if( Succeeded( hr ) ) then
        begin
        Result := True;
        Exit;
        end;
      end;
    end;

end;
//---------------------------------------------------------------------------
//-----------------------------------------------------------------
//-----------------------------------------------------------------
//-----------------------------------------------------------------
constructor TSampleGrabberCB.Create( AForm : TForm1 );
begin
  inherited Create();
  FForm := AForm;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB._AddRef() : Integer;
begin
  Result:= 2;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB._Release() : Integer;
begin
  Result:= 1;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB.QueryInterface(const IID: TGUID; out Obj): HRESULT;
begin
  Result:= E_NOINTERFACE;

  if( IsEqualGUID(IID, ISampleGrabberCB) or IsEqualGUID(IID, IUnknown) ) then
    if( GetInterface(IID, Obj) ) then
      Result:= S_OK;

end;
//-----------------------------------------------------------------
function TSampleGrabberCB.SampleCB(SampleTime: Double; pSample: IMediaSample): HResult;
begin
  FForm.SampleCB( SampleTime, pSample );

  Result:= S_OK;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB.BufferCB( SampleTime: Double; pBuffer: PByte; BufferLen: Integer ): HResult;
begin
  Result:= S_OK;
end;
//-----------------------------------------------------------------
initialization
  InitializeCriticalSection( GCriticalSection );

finalization
  DeleteCriticalSection( GCriticalSection );

end.
 