unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DirectShow9, StdCtrls, ExtCtrls;

const WM_NEW_FRAME = WM_APP + 100;

type
  TSampleGrabberCB = class;

  TForm1 = class(TForm)
    StartButton: TButton;
    StopButton: TButton;
    CapturedImage: TImage;
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FGraph              : IGraphBuilder;
    FMediaControl       : IMediaControl;
    FVideoSampleGrabber : ISampleGrabber;
    FVideoGrabberCB     : TSampleGrabberCB;
    FROTReg             : Integer;

    FBitmapInfoHeader   : TBitmapInfoHeader;
    FFrameBitmap        : TBitmap;
    
  private
    procedure WMNewFrame(var Message: TMessage); message WM_NEW_FRAME;
    
  private
    function  ConnectFilterToPin( ASourceFilter : IBaseFilter; ASinkPin : IPin ) : Boolean;
    
  public
    function SampleCB(SampleTime: Double; pSample: IMediaSample): HResult; stdcall;
    
  public
    { Public declarations }

  end;

  TSampleGrabberCB = class(TObject, IUnknown, ISampleGrabberCB)
  private
    FForm : TForm1;

  public
    // IUnknown
    function QueryInterface(const IID: TGUID; out Obj): HRESULT; stdcall;
    function _AddRef() : Integer; stdcall;
    function _Release() : Integer; stdcall;
    // ISampleGrabberCB
    function SampleCB(SampleTime: Double; pSample: IMediaSample): HResult; stdcall;
    function BufferCB(SampleTime: Double; pBuffer: PByte; BufferLen: longint): HResult; stdcall;
    
  public

    constructor Create( AForm : TForm1 );
    
  end;
  
var
  Form1: TForm1;

implementation

{$R *.dfm}

uses ActiveX;

var GCriticalSection  : TRTLCriticalSection;
var GHMem             : HGLOBAL;
var GFrameBuffer      : Pointer;
var GFormHandle       : HWND;

function DSGetPin( AFilter : IBaseFilter; APinDir : TPinDirection ) : IPin;
var
  hr          : HResult;
  APinEnum    : IEnumPins;
  Fetched     : ULONG;
  ADir        : TPinDirection;

begin
  Result := NIL;
  hr := AFilter.EnumPins( APinEnum );
  if( Failed( hr )) then
    Exit;

  while( Succeeded( APinEnum.Next( 1, Result, @Fetched ) ) ) do
    begin
    if( Fetched = 0 ) then
      Break;

    hr := Result.QueryDirection( ADir );
    if( Failed( hr )) then
      Continue;

    if( ADir = APinDir ) then
      Exit;

    end;

  Result := NIL;
end;

procedure DSFreeMediaType(mt: TAMMediaType);
begin
  if( mt.cbFormat <> 0 ) then
    begin
    CoTaskMemFree(mt.pbFormat);
    // Strictly unnecessary but tidier
    mt.cbFormat := 0;
    mt.pbFormat := NIL;
    end;

  mt.pUnk := NIL;
end;

function  TForm1.ConnectFilterToPin( ASourceFilter : IBaseFilter; ASinkPin : IPin ) : Boolean;
var
  hr          : HResult;
  AOutPin     : IPin;
  APinEnum    : IEnumPins;
  APinDir     : TPinDirection;
  Fetched     : ULONG;

begin
  Result := False;
  hr := ASourceFilter.EnumPins( APinEnum );
  if( Failed( hr )) then
    Exit;

  while( Succeeded( APinEnum.Next( 1, AOutPin, @Fetched ) ) ) do
    begin
    if( Fetched = 0 ) then
      Exit;

    hr := AOutPin.QueryDirection( APinDir );
    if( Failed( hr )) then
      Continue;

    if( APinDir = PINDIR_OUTPUT ) then
      begin
      hr := FGraph.Connect( AOutPin, ASinkPin );
      if( Succeeded( hr ) ) then
        begin
        Result := True;
        Exit;
        end;
      end;
    end;

end;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  GFormHandle := Handle;
end;
//---------------------------------------------------------------------------
procedure TForm1.WMNewFrame(var Message: TMessage);
var
  AScanLine       : Pointer;

begin
  if( FFrameBitmap = NIL ) then
    Exit;

  EnterCriticalSection( GCriticalSection );

  // Copy the pixel data to the bitmap.
  AScanLine := FFrameBitmap.ScanLine[ FBitmapInfoHeader.biHeight - 1 ];
  Move( GFrameBuffer^, AScanLine^, FBitmapInfoHeader.biHeight * FBitmapInfoHeader.biWidth * FBitmapInfoHeader.biBitCount div 8 );
  
  LeaveCriticalSection( GCriticalSection );

  // Copy the Frame bitmap to our improvised display.
  CapturedImage.Picture.Assign( FFrameBitmap );
end;
//---------------------------------------------------------------------------
procedure TForm1.StartButtonClick(Sender: TObject);
var
  AGrabber      : IBaseFilter;
  AFileFilter   : IBaseFilter;
  AMediaType    : TAMMediaType;
  WideFileName  : WideString;
  hr            : HResult;

  WideName      : WideString;
  pROT          : IRunningObjectTable;
  Moniker       : IMoniker;
  AOutPin       : IPin;
  AInPin        : IPin;

begin
  FVideoGrabberCB := TSampleGrabberCB.Create( Self );

  CoCreateInstance( CLSID_FilterGraph, NIL, CLSCTX_INPROC, IGraphBuilder, FGraph );

  WideFileName := 'Demo.avi';
  hr := FGraph.AddSourceFilter( @( WideFileName[ 1 ] ), 'File Source Filter', AFileFilter );

  // Create a grabber
  CoCreateInstance(CLSID_SampleGrabber, NIL, CLSCTX_INPROC, IBaseFilter, AGrabber );
  FGraph.AddFilter( AGrabber, 'Video Grabber' );

  FVideoSampleGrabber := AGrabber as ISampleGrabber;
  FVideoSampleGrabber.SetBufferSamples(False);
  FVideoSampleGrabber.SetOneShot(False);
  FVideoSampleGrabber.SetCallback( FVideoGrabberCB, 0);

  // Set the grabber's Media Format
  ZeroMemory(@AMediaType, SizeOf(AMediaType));
  AMediaType.formattype := FORMAT_VideoInfo;
  AMediaType.majortype  := MEDIATYPE_Video;
  AMediaType.subtype := MEDIASUBTYPE_RGB24;

  FVideoSampleGrabber.SetMediaType( AMediaType );

  AInPin := DSGetPin( AGrabber, PINDIR_INPUT );

  ConnectFilterToPin( AFileFilter, AInPin );
  
  AOutPin := DSGetPin( AGrabber, PINDIR_OUTPUT );

  FGraph.Render( AOutPin );

  FGraph.QueryInterface( IMediaControl, FMediaControl );

  if ( Succeeded( GetRunningObjectTable(0, pROT))) then
    begin
    WideName := format('FilterGraph %8p pid %8x',[pointer(FGraph),GetCurrentProcessId()]);
    hr := CreateItemMoniker('!', PWideChar(WideName ), Moniker );
    pROT.Register(ROTFLAGS_REGISTRATIONKEEPSALIVE, FGraph, Moniker, FROTReg );
    end;

  FVideoSampleGrabber.GetConnectedMediaType( AMediaType );
  
  ZeroMemory(@FBitmapInfoHeader, SizeOf(FBitmapInfoHeader));
  
  if( IsEqualGUID( AMediaType.FormatType, FORMAT_VideoInfo2) ) then
    FBitmapInfoHeader := PVideoInfoHeader2( AMediaType.pbFormat ).bmiHeader

  else if( IsEqualGUID( AMediaType.FormatType, FORMAT_VideoInfo) ) then
    FBitmapInfoHeader := PVideoInfoHeader(AMediaType.pbFormat ).bmiHeader

  else
    begin
    // Can't obtain the video format.
    StopButtonClick( NIL );
    DSFreeMediaType( AMediaType );
    Exit;
    end;

  DSFreeMediaType( AMediaType );

  FFrameBitmap := TBitmap.Create();
  FFrameBitmap.Width := FBitmapInfoHeader.biWidth;
  FFrameBitmap.Height := FBitmapInfoHeader.biHeight;

  CapturedImage.Width := FBitmapInfoHeader.biWidth;
  CapturedImage.Height := FBitmapInfoHeader.biHeight;

  case FBitmapInfoHeader.biBitCount of
    8 : FFrameBitmap.PixelFormat := pf8bit;
    16 : FFrameBitmap.PixelFormat := pf16bit;
    24 : FFrameBitmap.PixelFormat := pf24bit;
    else
      FFrameBitmap.PixelFormat := pf32bit;

  end;

  FMediaControl.Run();

  StartButton.Enabled := False;
  StopButton.Enabled := True;
end;
//---------------------------------------------------------------------------
procedure TForm1.StopButtonClick(Sender: TObject);
var
  pROT : IRunningObjectTable;

begin
  if( FMediaControl = NIL ) then
    Exit;
    
  if( Succeeded( GetRunningObjectTable(0, pROT))) then
    pROT.Revoke( FROTReg );
  
  FMediaControl.Stop();

  StartButton.Enabled := True;
  StopButton.Enabled := False;

  FMediaControl := NIL;
  FVideoSampleGrabber := NIL;
  FGraph := NIL;
  FVideoGrabberCB.Free();

  FFrameBitmap.Free();
  FFrameBitmap := NIL;
  
  if( GFrameBuffer <> NIL ) then
    begin
    // Release the old frame buffer if present.
    GlobalUnlock( GHMem );
    GlobalFree( GHMem );
    end;
    
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  StopButtonClick( NIL );
end;
//-----------------------------------------------------------------
function TForm1.SampleCB(SampleTime: Double; pSample: IMediaSample): HResult;
var
  ABufferLen : Cardinal;
  pBuffer    : PByte;

begin
  EnterCriticalSection( GCriticalSection );

  if( GFormHandle <> 0 ) then
    begin
    if( GFrameBuffer <> NIL ) then
      begin
      // Release the old frame buffer if present.
      GlobalUnlock( GHMem );
      GlobalFree( GHMem );
      end;

    ABufferLen := pSample.GetActualDataLength(); //GetSize();

    // Allocate new frame buffer.
    GHMem := GlobalAlloc( GMEM_FIXED, ABufferLen );
    GFrameBuffer := GlobalLock( GHMem );

    pSample.GetPointer( pBuffer );
    
    // Copy the data to the frame buffer.
    CopyMemory( GFrameBuffer, pBuffer, ABufferLen );

    LeaveCriticalSection( GCriticalSection );
    PostMessage( GFormHandle, WM_NEW_FRAME, 0, 0 );
    end;

  Result := 1;
end;
//-----------------------------------------------------------------
//-----------------------------------------------------------------
//-----------------------------------------------------------------
//-----------------------------------------------------------------
constructor TSampleGrabberCB.Create( AForm : TForm1 );
begin
  inherited Create();
  FForm := AForm;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB._AddRef() : Integer;
begin
  Result:= 2;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB._Release() : Integer;
begin
  Result:= 1;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB.QueryInterface(const IID: TGUID; out Obj): HRESULT;
begin
  Result:= E_NOINTERFACE;

  if( IsEqualGUID(IID, ISampleGrabberCB) or IsEqualGUID(IID, IUnknown) ) then
    if( GetInterface(IID, Obj) ) then
      Result:= S_OK;

end;
//-----------------------------------------------------------------
function TSampleGrabberCB.SampleCB(SampleTime: Double; pSample: IMediaSample): HResult;
begin
  FForm.SampleCB( SampleTime, pSample );

  Result:= S_OK;
end;
//-----------------------------------------------------------------
function TSampleGrabberCB.BufferCB( SampleTime: Double; pBuffer: PByte; BufferLen: Integer ): HResult;
begin
  Result:= S_OK;
end;
//-----------------------------------------------------------------
initialization
  InitializeCriticalSection( GCriticalSection );

finalization
  DeleteCriticalSection( GCriticalSection );

end.
 