unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DirectShow9, StdCtrls;

type
  TForm1 = class(TForm)
    StartButton: TButton;
    StopButton: TButton;
    procedure StartButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FGraph              : IGraphBuilder;
    FGraphBuilder       : ICaptureGraphBuilder2;
    FVideoCapFilter     : IBaseFilter;
    FMediaControl       : IMediaControl;
    
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses ActiveX;

const
  DefaultVideoDevice = '@device:*:{860BB310-5D01-11d0-BD3B-00A0C911CE86}';
  
procedure TForm1.StartButtonClick(Sender: TObject);
var
  WideFileName  : WideString;
  pSink         : IFileSinkFilter;
  pWriterFilter : IBaseFilter;
  AVideoMoniker : IMoniker;
  pBindCtx      : IBindCtx;
  chEaten       : Longint;
  
begin
  // Create Graph Builder.
  CoCreateInstance( CLSID_FilterGraph, NIL, CLSCTX_INPROC, IGraphBuilder, FGraph );

  // Create Capture Graph Builder.
  CoCreateInstance( CLSID_CaptureGraphBuilder2, NIL, CLSCTX_INPROC, ICaptureGraphBuilder2, FGraphBuilder );

  // Associate it with the Graph Builder.
  FGraphBuilder.SetFiltergraph( FGraph );

  // Get a Moniker for the default device.
  CreateBindCtx( 0, pBindCtx );
  MkParseDisplayName( pBindCtx, DefaultVideoDevice, chEaten, AVideoMoniker );
  pBindCtx := NIL;

  // Create a filter from the Moniker.
  AVideoMoniker.BindToObject(NIL, NIL, IBaseFilter, FVideoCapFilter);
  FGraph.AddFilter( FVideoCapFilter, 'Video Capture' );

  // Set the output file.
  WideFileName := 'Captured.avi';
  FGraphBuilder.SetOutputFileName( MEDIASUBTYPE_Avi, @( WideFileName[ 1 ] ), pWriterFilter, pSink );

  // Render the device to preview and file writer.
  FGraphBuilder.RenderStream( @PIN_CATEGORY_PREVIEW, @MEDIATYPE_Video, FVideoCapFilter, NIL, NIL );
  FGraphBuilder.RenderStream( @PIN_CATEGORY_CAPTURE, @MEDIATYPE_Video, FVideoCapFilter, NIL, pWriterFilter );

  // Get Media Control interface and start the capture.
  FGraph.QueryInterface( IMediaControl, FMediaControl );
  FMediaControl.Run();

  StartButton.Enabled := False;
  StopButton.Enabled := True;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  if( FMediaControl = NIL ) then
    Exit;
    
  // Stop the capture.
  FMediaControl.Stop();
  
  StartButton.Enabled := True;
  StopButton.Enabled := False;

  // Release the interfaces.
  FMediaControl := NIL;
  FVideoCapFilter := NIL;
  FGraphBuilder := NIL;
  FGraph := NIL;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  StopButtonClick( NIL );
end;

end.
